/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_kyocera.c,v 1.2 2005/01/27 23:55:18 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Kyocera camera maker-specific routines                             */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*                                                                    */
/* A bare minimum is known at this time (4/22/03)...just the PrintIM  */
/* tag.                                                               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Kyocera camera       */
/* models. This number is used to dispatch print and interpret        */
/* routines approopriate to the current image.                        */

int
kyocera_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
                case 'F':
                    if(strncmp(model,"FC-S3",4) == 0)
                        number = KYOCERA_FCS3;
                    else if(strncmp(model,"Finecam S5",10) == 0)
                        number = KYOCERA_FINECAMS5;
                break;
            default:
                break;
        }
    }
    return(number);
}


/* Dispatch a print routine for direct values in Kyocera cameras,     */
/* based upon model                                                   */

void
print_kyocera_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case GENERIC:
            default:
                print_kyocera1_makervalue(entry_ptr,make,model,prefix);
                kyocera1_interpret_value(entry_ptr);
                break;
        }
    }
}

/* Model-specific print routine for Kyocera cameras. This routine is  */
/* responsible for picking off any direct entry tags which are        */
/* peculiar and will not be handled properly by print_value()         */
/* (usually UNDEFINED values which fit in the 4-byte entry value). If */
/* there are no such entries, this function simply calls              */
/* print_value().                                                     */

void
print_kyocera1_makervalue(struct ifd_entry *entry_ptr,int make,int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,prefix);
                break;
        }
    }
}


/* Dispatch a routine to decode and print offset values for Kyocera   */
/* cameras.                                                           */

void
print_kyocera_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            /* currently only one routine available                   */
            default:
                kyocera1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
    }
}

/* Model-specific routine to print UNDEFINED values found at offsets  */
/* in Kyocera makernotes.                                             */

void
kyocera1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long value_offset;
    unsigned long dumplength;
    unsigned short marker;
    char *nameoftag;

    if(entry_ptr)
    {
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        value_offset = offset_ref + entry_ptr->value;

        switch(entry_ptr->tag)
        {
            case 0x0e00: /* PrintIM (Epson Print Image matching)      */
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu\n",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx\n",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu\n",value_offset);
                }
                process_pim(inptr,byteorder,entry_ptr->value,offset_ref,
                    entry_ptr->count,nameoftag,"Exif_Maker_",prefix,indent,addrwidth);
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}


/* Kyocera-specific tagnames for makernotes.                          */

char *
maker_kyocera_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        /* Add special-case models here                               */
        default:
            break;
    }

    /* If no model-specific tag is found, use a "generic"             */
    /* Kyocera tag                                                    */
    if(tagname == NULL)
    {
        switch(tag)
        {
            /* Haven't seen this yet, but leave it for now.           */
            case 0x0e00: tagname = "PrintIM"; break;
            default: break;
        }
    }
    return(tagname);
}



/* Report the "meaning" of tag values found at offsets in a           */
/* Kyocera MakerNote IFD entry (not at an offset).                    */

void
kyocera1_interpret_value(struct ifd_entry *entry_ptr)
{
/* nothing known yet                                                  */
}
